unit uMain;

{$mode objfpc}{$H+}

interface

uses
  SysUtils, Forms, StdCtrls, ExtCtrls, Buttons, uSaneScanner;

type

  { TfrmMain }

  TfrmMain = class(TForm)
    btnClose: TButton;
    gbxLibSane: TGroupBox;
    gbxDevice: TGroupBox;
    gbxLog : TGroupBox;
    lblLibSane: TLabel;
    lbxDevice : TListBox;
    mmoLog : TMemo;
    Panel1 : TPanel;
    pnlLeft: TPanel;
    sbnDevice: TSpeedButton;
    SpeedButton1 : TSpeedButton;
    procedure btnCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure lbxDeviceClick(Sender : TObject);
    procedure sbnDeviceClick(Sender: TObject);
    procedure AllDisabled;
    procedure SpeedButton1Click(Sender : TObject);
    procedure UpdateEnabled;
  private
    SaneScanner : TSaneScanner;
  public

  end;

var
  frmMain: TfrmMain;

implementation

{$R *.lfm}

uses
  process;

{ TfrmMain }

const
  capMainForm = 'Ein Zeitschriftenscanner';
  capActual   = 'Aktualiseren';
  capClose    = 'Beenden';
  capDevice   = 'Geräte';
  capLibSane  = 'LibSane';
  capLog      = 'Ereignisse (SaneScanner)';
  capNoDevice = 'Keine Geräte gefunden';
  capNoLoad   = 'Nicht geladen';
  capVersion  = 'Version';


procedure TfrmMain.FormCreate(Sender: TObject);
{*******************************************************************************
*  Erstellung des Formulars
}
begin
  //----------------------------
  // Formular
  Self.Caption := capMainForm;
  // Oben
  //----------------------------
  pnlLeft.Caption := '';
  // Libsane
  gbxLibSane.Caption := Format(' %s ', [capLibSane]);
  lblLibSane.Caption := capNoLoad;
  // Geräte
  gbxDevice.Caption  := Format(' %s ', [capDevice]);
  lbxDevice.Caption  := capActual;
  sbnDevice.ShowHint := True;
  // Schliessen
  btnClose.Caption := capClose;
  // Dateiformat und Log
  gbxLog.Caption := Format(' %s ', [capLog]);

  // verfügbarkeit
  AllDisabled;
  UpdateEnabled;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
{*******************************************************************************
*  Anwendung beenden.
}
begin
  if Assigned(SaneScanner)
  then SaneScanner.Destroy;
end;

procedure TfrmMain.lbxDeviceClick(Sender : TObject);
begin
  // Auswahl setzen
  SaneScanner.Device := lbxDevice.ItemIndex;
end;

procedure TfrmMain.btnCloseClick(Sender: TObject);
{*******************************************************************************
*  Anwendung beenden.
}
begin
  Self.Close;
end;

procedure TfrmMain.sbnDeviceClick(Sender: TObject);
{*******************************************************************************
*  Die verfügbaren Geräte ermitteln.
}
begin
  // Alle Bedienelemente deaktivieren
  AllDisabled;

  // Schliessen
  if Assigned(SaneScanner)
  then SaneScanner.Destroy;

  // Scanner - neu erstellen.
  SaneScanner        := TSaneScanner.Create;
  SaneScanner.Memo   := mmoLog;
  lblLibSane.Caption := Format('%s: %s', [capVersion, SaneScanner.SaneVersion]);

  // Liste initialisieren
  lbxDevice.Items.clear;
  lbxDevice.ItemIndex := -1;
  // Scanner vorhanden?
  if SaneScanner.FoundDevices
  then begin
    // Geräteliste holen.
    SaneScanner.ScannerList(lbxDevice.Items);
  end
  else lbxDevice.Caption := capNoDevice;
  // Oberfläche einstellen und Optionen des Gerätes holen.
  UpdateEnabled;
end;

procedure TfrmMain.AllDisabled;
{*******************************************************************************
*  Alle Bedien- und abhängigen Elemente verstecken.
}
begin
  // Geräte
  sbnDevice.Enabled := False;

  Application.ProcessMessages;
end;

procedure TfrmMain.SpeedButton1Click(Sender : TObject);
{*******************************************************************************
*  Memo leeren
}
begin
  mmoLog.Clear;
end;

procedure TfrmMain.UpdateEnabled;
{*******************************************************************************
*  Überprüfen, ob die Komponenten verfügbar sein sollen.
}
begin
  Application.ProcessMessages;

  // Geräte
  sbnDevice.Enabled := Length(lblLibSane.Caption) > (Length(capVersion) + 2);

  Application.ProcessMessages;
end;

end.

